import { __decorate, __param } from 'vscode/external/tslib/tslib.es6.js';
import { localizeWithPath } from 'vscode/vscode/vs/nls';
import Severity$1 from 'vscode/vscode/vs/base/common/severity';
import { Markers } from 'vscode/vscode/vs/workbench/contrib/markers/common/markers';
import { ITaskService } from 'vscode/vscode/vs/workbench/contrib/tasks/common/taskService.service';
import { IConfigurationService } from 'vscode/vscode/vs/platform/configuration/common/configuration.service';
import { TaskEventKind } from 'vscode/vscode/vs/workbench/contrib/tasks/common/tasks';
import { IDialogService } from 'vscode/vscode/vs/platform/dialogs/common/dialogs.service';
import { MarkerSeverity } from 'vscode/vscode/vs/platform/markers/common/markers';
import { IMarkerService } from 'vscode/vscode/vs/platform/markers/common/markers.service';
import { IViewsService } from 'vscode/vscode/vs/workbench/services/views/common/viewsService.service';
import { StorageScope, StorageTarget } from 'vscode/vscode/vs/platform/storage/common/storage';
import { IStorageService } from 'vscode/vscode/vs/platform/storage/common/storage.service';
import { createErrorWithActions } from 'vscode/vscode/vs/base/common/errorMessage';
import { Action } from 'vscode/vscode/vs/base/common/actions';
import { DEBUG_CONFIGURE_COMMAND_ID, DEBUG_CONFIGURE_LABEL } from 'vscode/vscode/vs/workbench/contrib/debug/browser/debugCommands';
import { ICommandService } from 'vscode/vscode/vs/platform/commands/common/commands.service';

const _moduleId = "vs/workbench/contrib/debug/browser/debugTaskRunner";
function once(match, event) {
    return (listener, thisArgs = null, disposables) => {
        const result = event(e => {
            if (match(e)) {
                result.dispose();
                return listener.call(thisArgs, e);
            }
        }, null, disposables);
        return result;
    };
}
var TaskRunResult;
( ((function(TaskRunResult) {
    TaskRunResult[TaskRunResult["Failure"] = 0] = "Failure";
    TaskRunResult[TaskRunResult["Success"] = 1] = "Success";
})(TaskRunResult || (TaskRunResult = {}))));
const DEBUG_TASK_ERROR_CHOICE_KEY = 'debug.taskerrorchoice';
let DebugTaskRunner = class DebugTaskRunner {
    constructor(taskService, markerService, configurationService, viewsService, dialogService, storageService, commandService) {
        this.taskService = taskService;
        this.markerService = markerService;
        this.configurationService = configurationService;
        this.viewsService = viewsService;
        this.dialogService = dialogService;
        this.storageService = storageService;
        this.commandService = commandService;
        this.canceled = false;
    }
    cancel() {
        this.canceled = true;
    }
    async runTaskAndCheckErrors(root, taskId) {
        try {
            this.canceled = false;
            const taskSummary = await this.runTask(root, taskId);
            if (this.canceled || (taskSummary && taskSummary.exitCode === undefined)) {
                return TaskRunResult.Failure;
            }
            const errorCount = taskId ? this.markerService.read({ severities: MarkerSeverity.Error, take: 2 }).length : 0;
            const successExitCode = taskSummary && taskSummary.exitCode === 0;
            const failureExitCode = taskSummary && taskSummary.exitCode !== 0;
            const onTaskErrors = this.configurationService.getValue('debug').onTaskErrors;
            if (successExitCode || onTaskErrors === 'debugAnyway' || (errorCount === 0 && !failureExitCode)) {
                return TaskRunResult.Success;
            }
            if (onTaskErrors === 'showErrors') {
                await this.viewsService.openView(Markers.MARKERS_VIEW_ID, true);
                return Promise.resolve(TaskRunResult.Failure);
            }
            if (onTaskErrors === 'abort') {
                return Promise.resolve(TaskRunResult.Failure);
            }
            const taskLabel = typeof taskId === 'string' ? taskId : taskId ? taskId.name : '';
            const message = errorCount > 1
                ? ( localizeWithPath(_moduleId, 0, "Errors exist after running preLaunchTask '{0}'.", taskLabel))
                : errorCount === 1
                    ? ( localizeWithPath(_moduleId, 1, "Error exists after running preLaunchTask '{0}'.", taskLabel))
                    : taskSummary && typeof taskSummary.exitCode === 'number'
                        ? ( localizeWithPath(
                _moduleId,
                2,
                "The preLaunchTask '{0}' terminated with exit code {1}.",
                taskLabel,
                taskSummary.exitCode
            ))
                        : ( localizeWithPath(_moduleId, 3, "The preLaunchTask '{0}' terminated.", taskLabel));
            let DebugChoice;
            ( ((function(DebugChoice) {
                DebugChoice[DebugChoice["DebugAnyway"] = 1] = "DebugAnyway";
                DebugChoice[DebugChoice["ShowErrors"] = 2] = "ShowErrors";
                DebugChoice[DebugChoice["Cancel"] = 0] = "Cancel";
            })(DebugChoice || (DebugChoice = {}))));
            const { result, checkboxChecked } = await this.dialogService.prompt({
                type: Severity$1.Warning,
                message,
                buttons: [
                    {
                        label: ( localizeWithPath(_moduleId, 4, "&&Debug Anyway")),
                        run: () => DebugChoice.DebugAnyway
                    },
                    {
                        label: ( localizeWithPath(_moduleId, 5, "&&Show Errors")),
                        run: () => DebugChoice.ShowErrors
                    }
                ],
                cancelButton: {
                    label: ( localizeWithPath(_moduleId, 6, "Abort")),
                    run: () => DebugChoice.Cancel
                },
                checkbox: {
                    label: ( localizeWithPath(_moduleId, 7, "Remember my choice in user settings")),
                }
            });
            const debugAnyway = result === DebugChoice.DebugAnyway;
            const abort = result === DebugChoice.Cancel;
            if (checkboxChecked) {
                this.configurationService.updateValue('debug.onTaskErrors', result === DebugChoice.DebugAnyway ? 'debugAnyway' : abort ? 'abort' : 'showErrors');
            }
            if (abort) {
                return Promise.resolve(TaskRunResult.Failure);
            }
            if (debugAnyway) {
                return TaskRunResult.Success;
            }
            await this.viewsService.openView(Markers.MARKERS_VIEW_ID, true);
            return Promise.resolve(TaskRunResult.Failure);
        }
        catch (err) {
            const taskConfigureAction = this.taskService.configureAction();
            const choiceMap = JSON.parse(this.storageService.get(DEBUG_TASK_ERROR_CHOICE_KEY, StorageScope.WORKSPACE, '{}'));
            let choice = -1;
            let DebugChoice;
            ( ((function(DebugChoice) {
                DebugChoice[DebugChoice["DebugAnyway"] = 0] = "DebugAnyway";
                DebugChoice[DebugChoice["ConfigureTask"] = 1] = "ConfigureTask";
                DebugChoice[DebugChoice["Cancel"] = 2] = "Cancel";
            })(DebugChoice || (DebugChoice = {}))));
            if (choiceMap[err.message] !== undefined) {
                choice = choiceMap[err.message];
            }
            else {
                const { result, checkboxChecked } = await this.dialogService.prompt({
                    type: Severity$1.Error,
                    message: err.message,
                    buttons: [
                        {
                            label: ( localizeWithPath(_moduleId, 4, "&&Debug Anyway")),
                            run: () => DebugChoice.DebugAnyway
                        },
                        {
                            label: taskConfigureAction.label,
                            run: () => DebugChoice.ConfigureTask
                        }
                    ],
                    cancelButton: {
                        run: () => DebugChoice.Cancel
                    },
                    checkbox: {
                        label: ( localizeWithPath(_moduleId, 8, "Remember my choice for this task"))
                    }
                });
                choice = result;
                if (checkboxChecked) {
                    choiceMap[err.message] = choice;
                    this.storageService.store(DEBUG_TASK_ERROR_CHOICE_KEY, JSON.stringify(choiceMap), StorageScope.WORKSPACE, StorageTarget.MACHINE);
                }
            }
            if (choice === DebugChoice.ConfigureTask) {
                await taskConfigureAction.run();
            }
            return choice === DebugChoice.DebugAnyway ? TaskRunResult.Success : TaskRunResult.Failure;
        }
    }
    async runTask(root, taskId) {
        if (!taskId) {
            return Promise.resolve(null);
        }
        if (!root) {
            return Promise.reject(( (new Error(( localizeWithPath(
                _moduleId,
                9,
                "Task '{0}' can not be referenced from a launch configuration that is in a different workspace folder.",
                typeof taskId === 'string' ? taskId : taskId.type
            ))))));
        }
        const task = await this.taskService.getTask(root, taskId);
        if (!task) {
            const errorMessage = typeof taskId === 'string'
                ? ( localizeWithPath(_moduleId, 10, "Could not find the task '{0}'.", taskId))
                : ( localizeWithPath(_moduleId, 11, "Could not find the specified task."));
            return Promise.reject(createErrorWithActions(errorMessage, [( (new Action(
                DEBUG_CONFIGURE_COMMAND_ID,
                DEBUG_CONFIGURE_LABEL,
                undefined,
                true,
                () => this.commandService.executeCommand(DEBUG_CONFIGURE_COMMAND_ID)
            )))]));
        }
        let taskStarted = false;
        const getTaskKey = (t) => t.getKey() ?? t.getMapKey();
        const taskKey = getTaskKey(task);
        const inactivePromise = ( (new Promise((c) => once(e => {
            return (e.kind === TaskEventKind.Inactive
                || (e.kind === TaskEventKind.ProcessEnded && e.exitCode === undefined))
                && getTaskKey(e.__task) === taskKey;
        }, this.taskService.onDidStateChange)(e => {
            taskStarted = true;
            c(e.kind === TaskEventKind.ProcessEnded ? { exitCode: e.exitCode } : null);
        }))));
        const promise = this.taskService.getActiveTasks().then(async (tasks) => {
            if (tasks.find(t => getTaskKey(t) === taskKey)) {
                const busyTasks = await this.taskService.getBusyTasks();
                if (busyTasks.find(t => getTaskKey(t) === taskKey)) {
                    taskStarted = true;
                    return inactivePromise;
                }
                return Promise.resolve(null);
            }
            once(e => ((e.kind === TaskEventKind.Active) || (e.kind === TaskEventKind.DependsOnStarted)) && getTaskKey(e.__task) === taskKey, this.taskService.onDidStateChange)(() => {
                taskStarted = true;
            });
            const taskPromise = this.taskService.run(task);
            if (task.configurationProperties.isBackground) {
                return inactivePromise;
            }
            return taskPromise.then(x => x ?? null);
        });
        return (
             (new Promise((c, e) => {
                const waitForInput = ( (new Promise(
                    resolve => once(e => (e.kind === TaskEventKind.AcquiredInput) && getTaskKey(e.__task) === taskKey, this.taskService.onDidStateChange)(() => {
                        resolve();
                    })
                )));
                promise.then(result => {
                    taskStarted = true;
                    c(result);
                }, error => e(error));
                waitForInput.then(() => {
                    const waitTime = task.configurationProperties.isBackground ? 5000 : 10000;
                    setTimeout(() => {
                        if (!taskStarted) {
                            const errorMessage = typeof taskId === 'string'
                                ? ( localizeWithPath(
                                _moduleId,
                                12,
                                "The task '{0}' cannot be tracked. Make sure to have a problem matcher defined.",
                                taskId
                            ))
                                : ( localizeWithPath(
                                _moduleId,
                                13,
                                "The task '{0}' cannot be tracked. Make sure to have a problem matcher defined.",
                                JSON.stringify(taskId)
                            ));
                            e({ severity: Severity$1.Error, message: errorMessage });
                        }
                    }, waitTime);
                });
            }))
        );
    }
};
DebugTaskRunner = ( (__decorate([
    ( (__param(0, ITaskService))),
    ( (__param(1, IMarkerService))),
    ( (__param(2, IConfigurationService))),
    ( (__param(3, IViewsService))),
    ( (__param(4, IDialogService))),
    ( (__param(5, IStorageService))),
    ( (__param(6, ICommandService)))
], DebugTaskRunner)));

export { DebugTaskRunner, TaskRunResult };
