import { __decorate, __param } from 'vscode/external/tslib/tslib.es6.js';
import { DisposableStore, dispose } from 'vscode/vscode/vs/base/common/lifecycle';
import './media/debugViewlet.css.js';
import { localizeWithPath, localize2WithPath } from 'vscode/vscode/vs/nls';
import { createActionViewItem } from 'vscode/vscode/vs/platform/actions/browser/menuEntryActionViewItem';
import { MenuRegistry, MenuId, registerAction2, Action2 } from 'vscode/vscode/vs/platform/actions/common/actions';
import { IConfigurationService } from 'vscode/vscode/vs/platform/configuration/common/configuration.service';
import { ContextKeyExpr } from 'vscode/vscode/vs/platform/contextkey/common/contextkey';
import { IContextKeyService } from 'vscode/vscode/vs/platform/contextkey/common/contextkey.service';
import { IContextMenuService, IContextViewService } from 'vscode/vscode/vs/platform/contextview/browser/contextView.service';
import { IInstantiationService } from 'vscode/vscode/vs/platform/instantiation/common/instantiation';
import { IProgressService } from 'vscode/vscode/vs/platform/progress/common/progress.service';
import { IQuickInputService } from 'vscode/vscode/vs/platform/quickinput/common/quickInput.service';
import { IStorageService } from 'vscode/vscode/vs/platform/storage/common/storage.service';
import { ITelemetryService } from 'vscode/vscode/vs/platform/telemetry/common/telemetry.service';
import { IThemeService } from 'vscode/vscode/vs/platform/theme/common/themeService.service';
import { IWorkspaceContextService } from 'vscode/vscode/vs/platform/workspace/common/workspace.service';
import { ViewPaneContainer, ViewsSubMenu } from 'vscode/vscode/vs/workbench/browser/parts/views/viewPaneContainer';
import { WorkbenchStateContext } from 'vscode/vscode/vs/workbench/common/contextkeys';
import { IViewDescriptorService } from 'vscode/vscode/vs/workbench/common/views.service';
import { IViewsService } from 'vscode/vscode/vs/workbench/services/views/common/viewsService.service';
import { StartDebugActionViewItem, FocusSessionActionViewItem } from './debugActionViewItems.js';
import { DEBUG_START_COMMAND_ID, FOCUS_SESSION_ID, STOP_ID, DISCONNECT_ID, DEBUG_START_LABEL, DEBUG_CONFIGURE_COMMAND_ID, DEBUG_CONFIGURE_LABEL, SELECT_AND_START_ID } from 'vscode/vscode/vs/workbench/contrib/debug/browser/debugCommands';
import { debugConfigure } from 'vscode/vscode/vs/workbench/contrib/debug/browser/debugIcons';
import { createDisconnectMenuItemAction } from './debugToolBar.js';
import { WelcomeView } from './welcomeView.js';
import { VIEWLET_ID, CONTEXT_DEBUG_UX_KEY, State, BREAKPOINTS_VIEW_ID, CONTEXT_DEBUG_UX, CONTEXT_DEBUG_STATE, getStateLabel, CONTEXT_DEBUGGERS_AVAILABLE, REPL_VIEW_ID } from 'vscode/vscode/vs/workbench/contrib/debug/common/debug';
import { IDebugService } from 'vscode/vscode/vs/workbench/contrib/debug/common/debug.service';
import { IExtensionService } from 'vscode/vscode/vs/workbench/services/extensions/common/extensions.service';
import { IWorkbenchLayoutService } from 'vscode/vscode/vs/workbench/services/layout/browser/layoutService.service';

const _moduleId = "vs/workbench/contrib/debug/browser/debugViewlet";
let DebugViewPaneContainer = class DebugViewPaneContainer extends ViewPaneContainer {
    constructor(layoutService, telemetryService, progressService, debugService, instantiationService, contextService, storageService, themeService, contextMenuService, extensionService, configurationService, contextViewService, contextKeyService, viewDescriptorService) {
        super(VIEWLET_ID, { mergeViewWithContainerWhenSingleView: true }, instantiationService, configurationService, layoutService, contextMenuService, telemetryService, extensionService, themeService, storageService, contextService, viewDescriptorService);
        this.progressService = progressService;
        this.debugService = debugService;
        this.contextViewService = contextViewService;
        this.contextKeyService = contextKeyService;
        this.paneListeners = ( (new Map()));
        this.stopActionViewItemDisposables = this._register(( (new DisposableStore())));
        this._register(this.debugService.onDidChangeState(state => this.onDebugServiceStateChange(state)));
        this._register(this.contextKeyService.onDidChangeContext(e => {
            if (e.affectsSome(( (new Set([CONTEXT_DEBUG_UX_KEY, 'inDebugMode']))))) {
                this.updateTitleArea();
            }
        }));
        this._register(this.contextService.onDidChangeWorkbenchState(() => this.updateTitleArea()));
        this._register(this.configurationService.onDidChangeConfiguration(e => {
            if (e.affectsConfiguration('debug.toolBarLocation') || e.affectsConfiguration('debug.hideLauncherWhileDebugging')) {
                this.updateTitleArea();
            }
        }));
    }
    create(parent) {
        super.create(parent);
        parent.classList.add('debug-viewlet');
    }
    focus() {
        super.focus();
        if (this.startDebugActionViewItem) {
            this.startDebugActionViewItem.focus();
        }
        else {
            this.focusView(WelcomeView.ID);
        }
    }
    getActionViewItem(action, options) {
        if (action.id === DEBUG_START_COMMAND_ID) {
            this.startDebugActionViewItem = this.instantiationService.createInstance(StartDebugActionViewItem, null, action, options);
            return this.startDebugActionViewItem;
        }
        if (action.id === FOCUS_SESSION_ID) {
            return (
                 (new FocusSessionActionViewItem(
                    action,
                    undefined,
                    this.debugService,
                    this.contextViewService,
                    this.configurationService
                ))
            );
        }
        if (action.id === STOP_ID || action.id === DISCONNECT_ID) {
            this.stopActionViewItemDisposables.clear();
            const item = this.instantiationService.invokeFunction(accessor => createDisconnectMenuItemAction(action, this.stopActionViewItemDisposables, accessor, { hoverDelegate: options.hoverDelegate }));
            if (item) {
                return item;
            }
        }
        return createActionViewItem(this.instantiationService, action, options);
    }
    focusView(id) {
        const view = this.getView(id);
        if (view) {
            view.focus();
        }
    }
    onDebugServiceStateChange(state) {
        if (this.progressResolve) {
            this.progressResolve();
            this.progressResolve = undefined;
        }
        if (state === State.Initializing) {
            this.progressService.withProgress({ location: VIEWLET_ID, }, _progress => {
                return (
                     (new Promise(resolve => this.progressResolve = resolve))
                );
            });
        }
    }
    addPanes(panes) {
        super.addPanes(panes);
        for (const { pane: pane } of panes) {
            if (pane.id === BREAKPOINTS_VIEW_ID) {
                this.breakpointView = pane;
                this.updateBreakpointsMaxSize();
            }
            else {
                this.paneListeners.set(pane.id, pane.onDidChange(() => this.updateBreakpointsMaxSize()));
            }
        }
    }
    removePanes(panes) {
        super.removePanes(panes);
        for (const pane of panes) {
            dispose(this.paneListeners.get(pane.id));
            this.paneListeners.delete(pane.id);
        }
    }
    updateBreakpointsMaxSize() {
        if (this.breakpointView) {
            const allOtherCollapsed = this.panes.every(view => !view.isExpanded() || view === this.breakpointView);
            this.breakpointView.maximumBodySize = allOtherCollapsed ? Number.POSITIVE_INFINITY : this.breakpointView.minimumBodySize;
        }
    }
};
DebugViewPaneContainer = ( (__decorate([
    ( (__param(0, IWorkbenchLayoutService))),
    ( (__param(1, ITelemetryService))),
    ( (__param(2, IProgressService))),
    ( (__param(3, IDebugService))),
    ( (__param(4, IInstantiationService))),
    ( (__param(5, IWorkspaceContextService))),
    ( (__param(6, IStorageService))),
    ( (__param(7, IThemeService))),
    ( (__param(8, IContextMenuService))),
    ( (__param(9, IExtensionService))),
    ( (__param(10, IConfigurationService))),
    ( (__param(11, IContextViewService))),
    ( (__param(12, IContextKeyService))),
    ( (__param(13, IViewDescriptorService)))
], DebugViewPaneContainer)));
MenuRegistry.appendMenuItem(MenuId.ViewContainerTitle, {
    when: ( (ContextKeyExpr.and(
         (ContextKeyExpr.equals('viewContainer', VIEWLET_ID)),
         (CONTEXT_DEBUG_UX.notEqualsTo('simple')),
         (WorkbenchStateContext.notEqualsTo('empty')),
         (ContextKeyExpr.or(
             (CONTEXT_DEBUG_STATE.isEqualTo('inactive')),
             (ContextKeyExpr.notEquals('config.debug.toolBarLocation', 'docked'))
        )),
         (ContextKeyExpr.or(
            ContextKeyExpr.not('config.debug.hideLauncherWhileDebugging'),
            ContextKeyExpr.not('inDebugMode')
        ))
    ))),
    order: 10,
    group: 'navigation',
    command: {
        precondition: ( (CONTEXT_DEBUG_STATE.notEqualsTo(getStateLabel(State.Initializing)))),
        id: DEBUG_START_COMMAND_ID,
        title: DEBUG_START_LABEL
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: DEBUG_CONFIGURE_COMMAND_ID,
            title: {
                value: DEBUG_CONFIGURE_LABEL,
                original: 'Open \'launch.json\'',
                mnemonicTitle: ( localizeWithPath(_moduleId, 0, "Open &&Configurations"))
            },
            metadata: {
                description: ( localize2WithPath(
                    _moduleId,
                    1,
                    'Opens the file used to configure how your program is debugged'
                ))
            },
            f1: true,
            icon: debugConfigure,
            precondition: ( (CONTEXT_DEBUG_UX.notEqualsTo('simple'))),
            menu: [{
                    id: MenuId.ViewContainerTitle,
                    group: 'navigation',
                    order: 20,
                    when: ( (ContextKeyExpr.and(
                         (ContextKeyExpr.equals('viewContainer', VIEWLET_ID)),
                         (CONTEXT_DEBUG_UX.notEqualsTo('simple')),
                         (WorkbenchStateContext.notEqualsTo('empty')),
                         (ContextKeyExpr.or(
                             (CONTEXT_DEBUG_STATE.isEqualTo('inactive')),
                             (ContextKeyExpr.notEquals('config.debug.toolBarLocation', 'docked'))
                        ))
                    )))
                }, {
                    id: MenuId.ViewContainerTitle,
                    order: 20,
                    when: ( (ContextKeyExpr.and(
                         (ContextKeyExpr.equals('viewContainer', VIEWLET_ID)),
                         (CONTEXT_DEBUG_STATE.notEqualsTo('inactive')),
                         (ContextKeyExpr.equals('config.debug.toolBarLocation', 'docked'))
                    )))
                }, {
                    id: MenuId.MenubarDebugMenu,
                    group: '2_configuration',
                    order: 1,
                    when: CONTEXT_DEBUGGERS_AVAILABLE
                }]
        });
    }
    async run(accessor) {
        const debugService = accessor.get(IDebugService);
        const quickInputService = accessor.get(IQuickInputService);
        const configurationManager = debugService.getConfigurationManager();
        let launch;
        if (configurationManager.selectedConfiguration.name) {
            launch = configurationManager.selectedConfiguration.launch;
        }
        else {
            const launches = configurationManager.getLaunches().filter(l => !l.hidden);
            if (launches.length === 1) {
                launch = launches[0];
            }
            else {
                const picks = ( (launches.map(l => ({ label: l.name, launch: l }))));
                const picked = await quickInputService.pick(picks, {
                    activeItem: picks[0],
                    placeHolder: ( localizeWithPath(
                        _moduleId,
                        2,
                        "Select a workspace folder to create a launch.json file in or add it to the workspace config file"
                    ))
                });
                if (picked) {
                    launch = picked.launch;
                }
            }
        }
        if (launch) {
            await launch.openConfigFile({ preserveFocus: false });
        }
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: 'debug.toggleReplIgnoreFocus',
            title: ( localizeWithPath(_moduleId, 3, "Debug Console")),
            toggled: ( (ContextKeyExpr.has(`view.${REPL_VIEW_ID}.visible`))),
            menu: [{
                    id: ViewsSubMenu,
                    group: '3_toggleRepl',
                    order: 30,
                    when: ( (ContextKeyExpr.and( (ContextKeyExpr.equals('viewContainer', VIEWLET_ID)))))
                }]
        });
    }
    async run(accessor) {
        const viewsService = accessor.get(IViewsService);
        if (viewsService.isViewVisible(REPL_VIEW_ID)) {
            viewsService.closeView(REPL_VIEW_ID);
        }
        else {
            await viewsService.openView(REPL_VIEW_ID);
        }
    }
});
MenuRegistry.appendMenuItem(MenuId.ViewContainerTitle, {
    when: ( (ContextKeyExpr.and(
         (ContextKeyExpr.equals('viewContainer', VIEWLET_ID)),
         (CONTEXT_DEBUG_STATE.notEqualsTo('inactive')),
         (ContextKeyExpr.or(
             (ContextKeyExpr.equals('config.debug.toolBarLocation', 'docked')),
             (ContextKeyExpr.has('config.debug.hideLauncherWhileDebugging'))
        ))
    ))),
    order: 10,
    command: {
        id: SELECT_AND_START_ID,
        title: ( localizeWithPath(_moduleId, 4, "Start Additional Session")),
    }
});

export { DebugViewPaneContainer };
